@@ -2,7 +2,7 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2018 Osimis S.A., Belgium
+ * Copyright (C) 2017-2019 Osimis S.A., Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Affero General Public License
@@ -26,9 +26,9 @@
 #include <boost/regex.hpp>
 #include <boost/lexical_cast.hpp>
 
-#include "Plugin.h"
 #include "DicomWebServers.h"
 
+#include <Plugins/Samples/Common/OrthancPluginCppWrapper.h>
 #include <Core/Toolbox.h>
 
 namespace OrthancPlugins
@@ -91,7 +91,6 @@ namespace OrthancPlugins
   static void ParseMultipartHeaders(bool& hasLength /* out */,
                                     size_t& length /* out */,
                                     std::string& contentType /* out */,
-                                    OrthancPluginContext* context,
                                     const char* startHeaders,
                                     const char* endHeaders)
   {
@@ -131,7 +130,7 @@ namespace OrthancPlugins
           }
           catch (boost::bad_lexical_cast&)
           {
-            OrthancPluginLogWarning(context, "Unable to parse the Content-Length of a multipart item");
+            LogWarning("Unable to parse the Content-Length of a multipart item");
           }
         }
         else if (key == "content-type")
@@ -144,7 +143,6 @@ namespace OrthancPlugins
 
 
   static const char* ParseMultipartItem(std::vector<MultipartItem>& result,
-                                        OrthancPluginContext* context,
                                         const char* start,
                                         const char* end,
                                         const boost::regex& nextSeparator)
@@ -167,7 +165,7 @@ namespace OrthancPlugins
     bool hasLength;
     size_t length;
     std::string contentType;
-    ParseMultipartHeaders(hasLength, length, contentType, context, startHeaders, endHeaders);
+    ParseMultipartHeaders(hasLength, length, contentType, startHeaders, endHeaders);
 
     boost::cmatch separator;
 
@@ -200,7 +198,6 @@ namespace OrthancPlugins
 
 
   void ParseMultipartBody(std::vector<MultipartItem>& result,
-                          OrthancPluginContext* context,
                           const char* body,
                           const uint64_t bodySize,
                           const std::string& boundary)
@@ -239,7 +236,7 @@ namespace OrthancPlugins
         }
         else
         {
-          current = ParseMultipartItem(result, context, current + 2, end, nextSeparator);
+          current = ParseMultipartItem(result, current + 2, end, nextSeparator);
         }
       }
     }
@@ -252,8 +249,8 @@ namespace OrthancPlugins
   {
     if (value.type() != Json::objectValue)
     {
-      OrthancPlugins::Configuration::LogError("This is not a JSON object");
-      throw Orthanc::OrthancException(Orthanc::ErrorCode_BadFileFormat);
+      throw Orthanc::OrthancException(Orthanc::ErrorCode_BadFileFormat,
+                                      "This is not a JSON object");
     }
 
     if (!value.isMember(key))
@@ -265,9 +262,9 @@ namespace OrthancPlugins
 
     if (tmp.type() != Json::objectValue)
     {
-      OrthancPlugins::Configuration::LogError("The field \"" + key + "\" of a JSON object is "
-                                              "not a JSON associative array as expected");
-      throw Orthanc::OrthancException(Orthanc::ErrorCode_BadFileFormat);
+      throw Orthanc::OrthancException(Orthanc::ErrorCode_BadFileFormat,
+                                      "The field \"" + key + "\" of a JSON object is "
+                                      "not a JSON associative array as expected");
     }
 
     Json::Value::Members names = tmp.getMemberNames();
@@ -276,9 +273,9 @@ namespace OrthancPlugins
     {
       if (tmp[names[i]].type() != Json::stringValue)
       {
-        OrthancPlugins::Configuration::LogError("Some value in the associative array \"" + key + 
-                                                "\" is not a string as expected");
-        throw Orthanc::OrthancException(Orthanc::ErrorCode_BadFileFormat);
+        throw Orthanc::OrthancException(Orthanc::ErrorCode_BadFileFormat,
+                                        "Some value in the associative array \"" + key + 
+                                        "\" is not a string as expected");
       }
       else
       {
@@ -288,17 +285,36 @@ namespace OrthancPlugins
   }
 
 
+  bool ParseTag(Orthanc::DicomTag& target,
+                const std::string& name)
+  {
+    OrthancPluginDictionaryEntry entry;
+    
+    if (OrthancPluginLookupDictionary(OrthancPlugins::GetGlobalContext(), &entry, name.c_str()) == OrthancPluginErrorCode_Success)
+    {
+      target = Orthanc::DicomTag(entry.group, entry.element);
+      return true;
+    }
+    else
+    {
+      return false;
+    }
+  }
+
+
   namespace Configuration
   {
     // Assume Latin-1 encoding by default (as in the Orthanc core)
     static Orthanc::Encoding defaultEncoding_ = Orthanc::Encoding_Latin1;
-    static OrthancConfiguration configuration_;
+    static std::auto_ptr<OrthancConfiguration> configuration_;
 
 
-    void Initialize(OrthancPluginContext* context)
-    {      
-      OrthancPlugins::OrthancConfiguration global(context);
-      global.GetSection(configuration_, "DicomWeb");
+    void Initialize()
+    {
+      configuration_.reset(new OrthancConfiguration);
+      
+      OrthancPlugins::OrthancConfiguration global;
+      global.GetSection(*configuration_, "DicomWeb");
 
       std::string s;
       if (global.LookupStringValue(s, "DefaultEncoding"))
@@ -307,41 +323,39 @@ namespace OrthancPlugins
       }
 
       OrthancPlugins::OrthancConfiguration servers;
-      configuration_.GetSection(servers, "Servers");
+      configuration_->GetSection(servers, "Servers");
       OrthancPlugins::DicomWebServers::GetInstance().Load(servers.GetJson());
     }
 
 
-    OrthancPluginContext* GetContext()
-    {
-      return configuration_.GetContext();
-    }
-
-
     std::string GetStringValue(const std::string& key,
                                const std::string& defaultValue)
     {
-      return configuration_.GetStringValue(key, defaultValue);
+      assert(configuration_.get() != NULL);
+      return configuration_->GetStringValue(key, defaultValue);
     }
 
 
     bool GetBooleanValue(const std::string& key,
                          bool defaultValue)
     {
-      return configuration_.GetBooleanValue(key, defaultValue);
+      assert(configuration_.get() != NULL);
+      return configuration_->GetBooleanValue(key, defaultValue);
     }
 
 
     unsigned int GetUnsignedIntegerValue(const std::string& key,
                                          unsigned int defaultValue)
     {
-      return configuration_.GetUnsignedIntegerValue(key, defaultValue);
+      assert(configuration_.get() != NULL);
+      return configuration_->GetUnsignedIntegerValue(key, defaultValue);
     }
 
 
     std::string GetRoot()
     {
-      std::string root = configuration_.GetStringValue("Root", "/dicom-web/");
+      assert(configuration_.get() != NULL);
+      std::string root = configuration_->GetStringValue("Root", "/dicom-web/");
 
       // Make sure the root URI starts and ends with a slash
       if (root.size() == 0 ||
@@ -361,7 +375,8 @@ namespace OrthancPlugins
 
     std::string GetWadoRoot()
     {
-      std::string root = configuration_.GetStringValue("WadoRoot", "/wado/");
+      assert(configuration_.get() != NULL);
+      std::string root = configuration_->GetStringValue("WadoRoot", "/wado/");
 
       // Make sure the root URI starts with a slash
       if (root.size() == 0 ||
@@ -382,8 +397,9 @@ namespace OrthancPlugins
 
     std::string  GetBaseUrl(const OrthancPluginHttpRequest* request)
     {
-      std::string host = configuration_.GetStringValue("Host", "");
-      bool ssl = configuration_.GetBooleanValue("Ssl", false);
+      assert(configuration_.get() != NULL);
+      std::string host = configuration_->GetStringValue("Host", "");
+      bool ssl = configuration_->GetBooleanValue("Ssl", false);
 
       if (host.empty() &&
           !LookupHttpHeader(host, request, "host"))
@@ -418,24 +434,6 @@ namespace OrthancPlugins
     }
 
 
-    void LogError(const std::string& message)
-    {
-      OrthancPluginLogError(GetContext(), message.c_str());
-    }
-
-
-    void LogWarning(const std::string& message)
-    {
-      OrthancPluginLogWarning(GetContext(), message.c_str());
-    }
-
-
-    void LogInfo(const std::string& message)
-    {
-      OrthancPluginLogInfo(GetContext(), message.c_str());
-    }
-
-
     Orthanc::Encoding GetDefaultEncoding()
     {
       return defaultEncoding_;
