@@ -2,7 +2,7 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2018 Osimis S.A., Belgium
+ * Copyright (C) 2017-2019 Osimis S.A., Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Affero General Public License
@@ -21,6 +21,7 @@
 
 #pragma once
 
+#include <Core/DicomFormat/DicomTag.h>
 #include <Core/Enumerations.h>
 
 #include <orthanc/OrthancCPlugin.h>
@@ -36,6 +37,14 @@
 
 namespace OrthancPlugins
 {
+  static const Orthanc::DicomTag DICOM_TAG_RETRIEVE_URL(0x0008, 0x1190);
+  static const Orthanc::DicomTag DICOM_TAG_FAILURE_REASON(0x0008, 0x1197);
+  static const Orthanc::DicomTag DICOM_TAG_WARNING_REASON(0x0008, 0x1196);
+  static const Orthanc::DicomTag DICOM_TAG_FAILED_SOP_SEQUENCE(0x0008, 0x1198);
+  static const Orthanc::DicomTag DICOM_TAG_REFERENCED_SOP_SEQUENCE(0x0008, 0x1199);
+  static const Orthanc::DicomTag DICOM_TAG_REFERENCED_SOP_CLASS_UID(0x0008, 0x1150);
+  static const Orthanc::DicomTag DICOM_TAG_REFERENCED_SOP_INSTANCE_UID(0x0008, 0x1155);
+
   struct MultipartItem
   {
     const char*   data_;
@@ -52,7 +61,6 @@ namespace OrthancPlugins
                         const std::string& header);
 
   void ParseMultipartBody(std::vector<MultipartItem>& result,
-                          OrthancPluginContext* context,
                           const char* body,
                           const uint64_t bodySize,
                           const std::string& boundary);
@@ -61,12 +69,13 @@ namespace OrthancPlugins
                              const Json::Value& value,
                              const std::string& key);
 
+  bool ParseTag(Orthanc::DicomTag& target,
+                const std::string& name);
+
   namespace Configuration
   {
-    void Initialize(OrthancPluginContext* context);
+    void Initialize();
 
-    OrthancPluginContext* GetContext();
-    
     std::string GetStringValue(const std::string& key,
                                const std::string& defaultValue);
 
@@ -87,12 +96,6 @@ namespace OrthancPlugins
                            const std::string& seriesInstanceUid,
                            const std::string& sopInstanceUid);
 
-    void LogError(const std::string& message);
-
-    void LogWarning(const std::string& message);
-
-    void LogInfo(const std::string& message);
-
     Orthanc::Encoding GetDefaultEncoding();
   }
 }
