@@ -2,7 +2,7 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2018 Osimis S.A., Belgium
+ * Copyright (C) 2017-2019 Osimis S.A., Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Affero General Public License
@@ -56,8 +56,8 @@ namespace OrthancPlugins
 
         for (size_t i = 0; i < members.size(); i++)
         {
-          std::auto_ptr<Orthanc::WebServiceParameters> parameters(new Orthanc::WebServiceParameters);
-          parameters->FromJson(servers[members[i]]);
+          std::auto_ptr<Orthanc::WebServiceParameters> parameters
+            (new Orthanc::WebServiceParameters(servers[members[i]]));
 
           servers_[members[i]] = parameters.release();
         }
@@ -65,15 +65,16 @@ namespace OrthancPlugins
     }
     catch (Orthanc::OrthancException& e)
     {
-      OrthancPlugins::Configuration::LogError("Exception while parsing the \"DicomWeb.Servers\" section "
-                                              "of the configuration file: " + std::string(e.What()));
+      OrthancPlugins::LogError("Exception while parsing the \"DicomWeb.Servers\" section "
+                               "of the configuration file: " + std::string(e.What()));
       throw;
     }
 
     if (!ok)
     {
-      OrthancPlugins::Configuration::LogError("Cannot parse the \"DicomWeb.Servers\" section of the configuration file");
-      throw Orthanc::OrthancException(Orthanc::ErrorCode_BadFileFormat);
+      throw Orthanc::OrthancException(
+        Orthanc::ErrorCode_BadFileFormat,
+        "Cannot parse the \"DicomWeb.Servers\" section of the configuration file");
     }
   }
 
@@ -93,8 +94,8 @@ namespace OrthancPlugins
     if (server == servers_.end() ||
         server->second == NULL)
     {
-      OrthancPlugins::Configuration::LogError("Inexistent server: " + name);
-      throw Orthanc::OrthancException(Orthanc::ErrorCode_InexistentItem);
+      throw Orthanc::OrthancException(Orthanc::ErrorCode_InexistentItem,
+                                      "Inexistent server: " + name);
     }
     else
     {
@@ -154,18 +155,32 @@ namespace OrthancPlugins
       url += uri;
     }
 
-    std::vector<const char*> httpHeadersKeys(httpHeaders.size());
-    std::vector<const char*> httpHeadersValues(httpHeaders.size());
+    std::map<std::string, std::string> allHttpHeaders = server.GetHttpHeaders();
+    
+    {
+      // Add the user-specified HTTP headers to the HTTP headers
+      // coming from the Orthanc configuration file
+      for (std::map<std::string, std::string>::const_iterator
+             it = httpHeaders.begin(); it != httpHeaders.end(); ++it)
+      {
+        allHttpHeaders[it->first] = it->second;
+      }
+    }
+
+    std::vector<const char*> httpHeadersKeys(allHttpHeaders.size());
+    std::vector<const char*> httpHeadersValues(allHttpHeaders.size());
 
     {
       size_t pos = 0;
       for (std::map<std::string, std::string>::const_iterator
-             it = httpHeaders.begin(); it != httpHeaders.end(); ++it)
+             it = allHttpHeaders.begin(); it != allHttpHeaders.end(); ++it)
       {
         httpHeadersKeys[pos] = it->first.c_str();
         httpHeadersValues[pos] = it->second.c_str();
         pos += 1;
       }
+
+      assert(pos == allHttpHeaders.size());
     }
 
     const char* bodyContent = NULL;
@@ -179,10 +194,10 @@ namespace OrthancPlugins
       bodySize = body.size();
     }
 
-    OrthancPluginContext* context = OrthancPlugins::Configuration::GetContext();
+    OrthancPluginContext* context = OrthancPlugins::GetGlobalContext();
 
     uint16_t status = 0;
-    MemoryBuffer answerHeadersTmp(context);
+    MemoryBuffer answerHeadersTmp;
     OrthancPluginErrorCode code = OrthancPluginHttpClient(
       context, 
       /* Outputs */
@@ -190,7 +205,7 @@ namespace OrthancPlugins
       method,
       url.c_str(), 
       /* HTTP headers*/
-      httpHeaders.size(),
+      allHttpHeaders.size(),
       httpHeadersKeys.empty() ? NULL : &httpHeadersKeys[0],
       httpHeadersValues.empty() ? NULL : &httpHeadersValues[0],
       bodyContent, bodySize,
@@ -205,9 +220,10 @@ namespace OrthancPlugins
     if (code != OrthancPluginErrorCode_Success ||
         (status < 200 || status >= 300))
     {
-      OrthancPlugins::Configuration::LogError("Cannot issue an HTTP query to " + url + 
-                                              " (HTTP status: " + boost::lexical_cast<std::string>(status) + ")");
-      throw Orthanc::OrthancException(static_cast<Orthanc::ErrorCode>(code));
+      throw Orthanc::OrthancException(
+        static_cast<Orthanc::ErrorCode>(code),
+        "Cannot issue an HTTP query to " + url + 
+        " (HTTP status: " + boost::lexical_cast<std::string>(status) + ")");
     }
 
     Json::Value json;
@@ -242,9 +258,9 @@ namespace OrthancPlugins
   {
     if (resource.find('?') != std::string::npos)
     {
-      OrthancPlugins::Configuration::LogError("The GET arguments must be provided in a separate field "
-                                              "(explicit \"?\" is disallowed): " + resource);
-      throw Orthanc::OrthancException(Orthanc::ErrorCode_BadFileFormat);
+      throw Orthanc::OrthancException(Orthanc::ErrorCode_BadFileFormat,
+                                      "The GET arguments must be provided in a separate field "
+                                      "(explicit \"?\" is disallowed): " + resource);
     }
 
     uri = resource;
